==> jamesmcdonald-cpanm-0.9.10.tar.gz: jamesmcdonald-cpanm-0.9.10/lib/puppet/provider/cpanm/default.rb <==
Puppet::Type.type(:cpanm).provide(:default) do
  desc 'Manage CPAN modules with cpanm'

  commands :cpanm => 'cpanm'
  commands :perl => 'perl'
  commands :perldoc => 'perldoc'

  # Override the `commands`-generated cpanm method to avoid resetting locale to 'C'
  # Avoids triggering an old Encode bug in MakeMakeFiles for some modules
  def cpanm(*args)
    result = `cpanm #{args.join ' '}`
    if $? != 0
      raise Puppet::ExecutionFailure, result
    end
    return result
  end

  def latest?
    begin
      installed=perl "-m#{@resource[:name]}", "-eprint $#{@resource[:name]}::VERSION",  '2>/dev/null'
    rescue Puppet::ExecutionFailure
      installed=''
    end
    options = []
    if @resource[:mirror]
      options << "--from"
      options << @resource[:mirror]
    end
    options << '--info'
    options << @resource[:name]

    cpan=cpanm(options).split("\n")[-1].match(/([0-9]+\.?[0-9]*).tar.gz/)
    if cpan
      latest = cpan[1]
      Puppet.debug("Installed: #{installed}, CPAN: #{latest}")
      if latest > installed
        return false
      end
    end
    return true
  end

  def create
    options = []

    if @resource[:mirror]
      options << "--from"
      options << @resource[:mirror]
    end

    if @resource[:force] == :true
      options << "-f"
    end

    if @resource[:test] == :false
      options << "-n"
    end

    options << @resource[:name]

    cpanm(options)
  end

  #  alias update create
  def destroy
    begin
      cpanm '-U', '-f', @resource[:name]
    rescue Puppet::ExecutionFailure
      #error = Puppet::Error.new("Failed to remove CPAN package: #{e}")
      #error.set_backtrace(e.backtrace)
      #raise error
    end
  end

  def exists?
    begin
      perl "-m#{@resource[:name]}", '-e1', '>/dev/null', '2>&1'
    rescue Puppet::ExecutionFailure
      false
    end
  end

  def self.instances
    modules = {}
    name = nil
    perldoc('-tT', 'perllocal').split("\n").each do |r|
      if r.include?('"Module"') then
        name = r.split[-1]
        modules[name] = new(:name => name)
      end
      if r.include?('VERSION: ') and name
        r.split[-1].delete('"')
        #modules[name].version = version
      end
    end
    modules.map do |k,v| v end
  end
end

==> jamesmcdonald-cpanm-0.9.17.tar.gz: jamesmcdonald-cpanm-0.9.17/lib/puppet/provider/cpanm/default.rb <==
Puppet::Type.type(:cpanm).provide(:default) do
  desc 'Manage CPAN modules with cpanm'

  commands :cpanm => 'cpanm'
  commands :perl => 'perl'
  commands :perldoc => 'perldoc'

  # Override the `commands`-generated cpanm method to avoid resetting locale to 'C'
  # Avoids triggering an old Encode bug in MakeMakeFiles for some modules
  def cpanm(*args)
    result = `cpanm #{args.join ' '}`
    if $? != 0
      raise Puppet::ExecutionFailure, result
    end
    return result
  end

  def latest?
    begin
      name = @resource[:name]
      name = name.split('@')[0]
      installed = `perl -e 'require Module::Metadata; $meta = Module::Metadata->new_from_module("#{name}"); if ($meta) {print $meta->version} else {exit 1}'`
    rescue Puppet::ExecutionFailure
      installed=''
    end
    options = []
    if @resource[:mirror]
      options << "--from"
      options << @resource[:mirror]
    end
    options << '--info'
    options << @resource[:name]

    cpan=cpanm(options).split("\n")[-1].match(/([0-9]+\.?[0-9]*).tar.gz/)
    if cpan
      latest = cpan[1]
      Puppet.debug("Installed: #{installed}, CPAN: #{latest}")
      if latest > installed
        return false
      end
    end
    return true
  end

  def create
    options = []

    if @resource[:mirror]
      options << "--from"
      options << @resource[:mirror]
    end

    if @resource[:force] == :true || @resource[:force] == true
      options << "-f"
    end

    if @resource[:test] == :false || @resource[:test] == false
      options << "-n"
    end

    options << @resource[:name]

    cpanm(options)
  end

  #  alias update create
  def destroy
    begin
      cpanm '-U', '-f', @resource[:name]
    rescue Puppet::ExecutionFailure
      #error = Puppet::Error.new("Failed to remove CPAN package: #{e}")
      #error.set_backtrace(e.backtrace)
      #raise error
    end
  end

  def exists?
    begin
      name = @resource[:name]
      name, version = name.split('@')
      installed = `perl -e 'require Module::Metadata; $meta = Module::Metadata->new_from_module("#{name}"); if ($meta) {print $meta->version} else {exit 1}'`
      if $? != 0
        raise Puppet::ExecutionFailure, installed
      end
      if not version.nil? and not version.empty?
        if installed.eql? version
          Puppet.debug("Debugging message - versions of #{name} are identical: installed:#{installed} vs requested:#{version}")
          true
        else
          Puppet.debug("Debugging message - versions of #{name} are different: installed:#{installed} vs requested:#{version}")
          false
        end
      else
        true
      end
    rescue Puppet::ExecutionFailure
      false
    end
  end

  def self.instances
    modules = {}
    name = nil
    perldoc('-tT', 'perllocal').split("\n").each do |r|
      if r.include?('"Module"') then
        name = r.split[-1]
        modules[name] = new(:name => name)
      end
      if r.include?('VERSION: ') and name
        r.split[-1].delete('"')
        #modules[name].version = version
      end
    end
    modules.map do |k,v| v end
  end
end
